import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class Textsuche extends JFrame {
	
	private JPanel p1, p2;
	private JTextArea ta;
	private JLabel t;
	private JTextField eingabe;
	private JButton suche;
	private JComboBox algorithmus;
	
	
	public Textsuche() {
		super("Textsuche");
		this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
		
		Container framec = getContentPane();
		framec.setLayout(new BorderLayout());
		
		// Erzeugung und Layout des Fensters
        p1 = new JPanel();
        p2 = new JPanel();
        p1.setLayout(new BorderLayout());
        p2.setLayout(new GridLayout(1,4));
		
	 	ta = new JTextArea("Hoola-Hoola girls like Hooligans.");
	 	ta.setLineWrap(true);
	 	t = new JLabel("Suchen nach:   ");
	 	t.setHorizontalAlignment(JLabel.RIGHT);
	 	eingabe = new JTextField("Hooligan");
	 	String[] s = {"Boyer-Moore", "Knuth-Morris-Pratt"};
	 	algorithmus = new JComboBox(s);
	 	
	 	suche = new JButton("Suche");
	 	suche.addActionListener(new ActionListener() {
	 		public void actionPerformed(ActionEvent e) {
	 			String s = eingabe.getText();
	 			String text = ta.getText();
	 			int i = -1;
	 			// Aufrufen des Boyer-Moore-Algorithmus
	 			if (!(s.equals(""))) {
	 				if (algorithmus.getSelectedIndex() == 0) {
	 					// Prfen, ob Boyer-Moore-Alg. ausgewhlt wurde
	 					i = boyermoore(text, s);
	 				}
	 				else {
	 					// sonst starte den KMP
	 					i = kmp(text, s);
	 				}
	 				if (i >= 0) {
			 			// Wenn Zeichenkette gefunden, dann im Text markieren
	 					ta.select(i, i + s.length());
	 				}
	 				else {
			 			// nicht gefunden, Warndialog einblenden
	 					JOptionPane.showMessageDialog(null,"String nicht gefunden!", "",JOptionPane.WARNING_MESSAGE);
		 			}
		 		}
		 		// Focus auf TextArea setzen, damit Markierung sichtbar wird
				ta.requestFocus();
	 		}
	 	});
	 	
        // Komponenten den Panels hinzufuegen                
        p1.add(ta, BorderLayout.CENTER);
        p2.add(algorithmus);
        p2.add(t);
        p2.add(eingabe);
        p2.add(suche);
        
        // Panels dem Fenster hinzufuegen
        framec.add(p1, BorderLayout.CENTER);
        framec.add(p2, BorderLayout.SOUTH);
        this.setSize(new Dimension(500, 300));
        this.setVisible(true);
	}
	
	/** Methode, welche den Knuth-Morris-Pratt-Algorithmus implementiert.
	  * @return Anfangsposition des gefunden Strings, -1 wenn nicht gefunden
	  * @ param text: der zu durchsuchende Text
	  * @ param s: String der in text zu suchen ist
	  */	
	public int kmp(String text, String s) {
		int i, j, k;
		int vergleiche = 0;
		int m = s.length();
		int n = text.length();
		int[] d = new int[m];
		
		if (n < m) {
			return -1;
		}
		j = 0;
		k = -1;
		d[0] = -1;
		while (j < m - 1) {
			while ((k >= 0) && (s.charAt(j) != s.charAt(k))) {
				k = d[k];
			}
			j++;
			k++;
			if (s.charAt(j) == s.charAt(k)) {
				d[j] = d[k];
			}
			else {
				d[j] = k;
			}
		}
		for (int z = 0; z < d.length; z++) {
			System.out.print(d[z] + " ");
		}
		System.out.println();
		i = 0;
		j = 0;
		while ((j < m) && (i < n)) {
			while ((j >=0) && (text.charAt(i) != s.charAt(j))) {
				j = d[j];
				vergleiche++;
			}
			if (j >= 0) { vergleiche++; }
			i++;
			j++;
		}
		//JOptionPane.showMessageDialog(null,"Knuth-Morris-Pratt Vergleiche: " + vergleiche);
		//System.out.println("KMP: Anzahl Vergleiche: " + vergleiche);
		if (j == m) {
			return (i-m);
		}
		else {
			return -1;
		}
	}
	
	/** Methode, welche den Boyer-Moore-Algorithmus implementiert.
	  * @return Anfangsposition des gefunden Strings, -1 wenn nicht gefunden
	  * @ param text: der zu durchsuchende Text
	  * @ param s: String der in text zu suchen ist
	  */
	public int boyermoore(String text, String s) {
		int[] d;
		int i,j,k;
		int vergleiche = -1;
		int m = s.length();
		int n = text.length();
		
		if (n < m) {
			// Wenn Suchbegriff laenger als Text ist, dann abbrechen
			return -1;
		}
		d = new int[256];
		// Zeichenarray initialisieren
		for (i = 0; i < 256; i++) {
			d[i] = m;
		}
		for (i = 0; i < s.length()-1; i++) {
			// Zeichenposition im Suchbegriff merken
			d[s.charAt(i)] = m-i-1;			
		}
		i = m;
		do {
			j = m;
			k = i;
			do {
				// Solange wiederholen, bis m Zeichen gleich sind oder
				// an einer Stelle die Zeichen verschieden sind
				k--;
				j--;
				vergleiche++;
			} while (!((j < 0) || (s.charAt(j) != text.charAt(k))));
			// Position der Maske neu bestimmen
			i = i + d[text.charAt(i-1)];
		} while (!((j < 0) || (i > n)));
		JOptionPane.showMessageDialog(null,"Boyer-Moore Vergleiche: " + vergleiche);
		// System.out.println("BM : Anzahl Vergleiche: " + vergleiche);
		if (j < 0) {
			// gefunden, also genaue Position berechnen und zurckgeben
			return (i-m-d[s.charAt(m-1)]);
		}
		else {
			// nicht gefunden
			return -1;
		}
	}
	
	public static void main(String args[]) {
		
		Textsuche fenster = new Textsuche();
		fenster.show();
	}
}
